package p2pcdnsim;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import p2pcdnsim.network.NetworkTopology;
import p2pcdnsim.network.TopologyConstructor;
import p2pcdnsim.report.Bandwidth;
import p2pcdnsim.report.SimReporter;
import p2pcdnsim.xml.XMLNetworkTopologyBuilder;

import desmoj.core.simulator.Experiment;
import desmoj.core.simulator.Model;
import desmoj.core.simulator.SimTime;
import desmoj.core.statistic.*;

public class SimP2P extends Model {

	public static final int SIM_TIME = 80;

	public SimP2P(Model owner, String modelName, boolean showInReport, boolean showInTrace) {
		super(owner, modelName, showInReport, showInTrace);
	}

	public String description() {
		return "P2P simulator";
	}

	public void doInitialSchedules() {

		try {
			TopologyConstructor tConstructor = new TopologyConstructor();
			tConstructor.setModel(this);
			tConstructor.setBuilder(new XMLNetworkTopologyBuilder(getRouterDoc(), "100000000", "0.000", null));
			tConstructor.constructTopology();
			NetworkTopology topology = tConstructor.getNetworkTopology();
			
			topology.printInfo();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		long startTime = System.currentTimeMillis();
		long finishTime;

		// create model and experiment
		SimP2P model = new SimP2P(null, "P2P Simulator", true, true);
		Experiment exp = new Experiment("Simulator");
		model.connectToExperiment(exp);

		// set experiment parameters
		exp.setShowProgressBar(true); // display a progress bar (or not)
		exp.stop(new SimTime(SIM_TIME)); // set end of simulation at 1500
		// time units
		exp.tracePeriod(new SimTime(0.0), new SimTime(1)); // set the period of
		// the trace
		exp.debugPeriod(new SimTime(0.0), new SimTime(1)); // and debug output

		exp.start();
		exp.report();
		exp.finish();

		finishTime = System.currentTimeMillis();
		long time = finishTime - startTime;

		Date date = new Date(time);
		String simulationTime = (new SimpleDateFormat("hh:mm:ss")).format(date);
		System.out.println("Simulation time: " + simulationTime);

	}

	public void init() {
		SimReporter.getInstance().registerReportable(SimReporter.NUMBER_OF_REQUESTS, new Count(this, "Number of Requests", true, false));
		SimReporter.getInstance().registerReportable(SimReporter.HIT_RATIO, new Count(this, "Hit Ratio", true, false));
		SimReporter.getInstance().registerReportable(SimReporter.MISS_RATIO, new Count(this, "Miss Ratio", true, false));		
		SimReporter.getInstance().registerReportable(SimReporter.STARTUP_DELAY, new Accumulate(this, "Startup Delay", true, false));
		SimReporter.getInstance().registerReportable(SimReporter.PACKET_LOSS, new Count(this, "Packet Loss", true, false));
		SimReporter.getInstance().registerReportable(SimReporter.BANDWIDTH_CONSUMPTION, new Bandwidth (this, "Bandwidth Consumption", true, false));
	}

	private String getScenario() {
		return "scenario1";
	}
	
	private Document getRouterDoc() throws JDOMException, IOException {
		String file = "xml/"+getScenario()+"/Routers.xml";
		SAXBuilder builder = new SAXBuilder();
		return builder.build(new File(file));
	}

}
